@@ -14,7 +14,7 @@ import tox
 from tox import venv
 from tox.config import parseconfig
 from tox.result import ResultLog
-from tox.session import Session, main
+from tox.session import Reporter, Session, main
 from tox.venv import CreationConfig, VirtualEnv, getdigest
 
 mark_dont_run_on_windows = pytest.mark.skipif(os.name == "nt", reason="non windows test")
@@ -69,8 +69,8 @@ def cmd(request, capfd, monkeypatch):
     request.addfinalizer(py.path.local().chdir)
 
     def run(*argv):
-        key = str(b"PYTHONPATH")
-        python_paths = (i for i in (str(os.getcwd()), os.getenv(key)) if i)
+        key = str("PYTHONPATH")
+        python_paths = (i for i in (os.getcwd(), os.getenv(key)) if i)
         monkeypatch.setenv(key, os.pathsep.join(python_paths))
 
         with RunResult(capfd, argv) as result:
@@ -128,6 +128,7 @@ class ReportExpectMock:
         self._calls = []
         self._index = -1
         self.session = session
+        self.orig_reporter = Reporter(session)
 
     def clear(self):
         self._calls[:] = []
@@ -136,8 +137,7 @@ class ReportExpectMock:
         if name[0] == "_":
             raise AttributeError(name)
         elif name == "verbosity":
-            # FIXME: special case for property on Reporter class, may it be generalized?
-            return 0
+            return self.orig_reporter.verbosity
 
         def generic_report(*args, **_):
             self._calls.append((name,) + args)
@@ -164,7 +164,7 @@ class ReportExpectMock:
     def expect(self, cat, messagepattern="*", invert=False):
         __tracebackhide__ = True
         if not messagepattern.startswith("*"):
-            messagepattern = "*" + messagepattern
+            messagepattern = "*{}".format(messagepattern)
         while self._index < len(self._calls):
             try:
                 call = self.getnext(cat)
@@ -403,30 +403,98 @@ def mock_venv(monkeypatch):
     Note: because we inherit, to keep things sane you must call the py environment and only that;
     and cannot install any packages. """
 
+    # first ensure we have a clean python path
+    monkeypatch.delenv("PYTHONPATH", raising=False)
+
+    # object to collect some data during the execution
+    class Result(object):
+        def __init__(self):
+            self.popens = []
+            self.cwd = None
+            self.session = None
+
+    res = Result()
+
+    # convince tox that the current running virtual environment is already the env we would create
     class ProxyCurrentPython:
         @classmethod
         def readconfig(cls, path):
-            assert path.dirname.endswith("{}py".format(os.sep))
-            return CreationConfig(
-                md5=getdigest(sys.executable),
-                python=sys.executable,
-                version=tox.__version__,
-                sitepackages=False,
-                usedevelop=False,
-                deps=[],
-                alwayscopy=False,
-            )
+            if path.dirname.endswith("{}py".format(os.sep)):
+                return CreationConfig(
+                    base_resolved_python_md5=getdigest(sys.executable),
+                    base_resolved_python_path=sys.executable,
+                    tox_version=tox.__version__,
+                    sitepackages=False,
+                    usedevelop=False,
+                    deps=[],
+                    alwayscopy=False,
+                )
+            elif path.dirname.endswith("{}.package".format(os.sep)):
+                return CreationConfig(
+                    base_resolved_python_md5=getdigest(sys.executable),
+                    base_resolved_python_path=sys.executable,
+                    tox_version=tox.__version__,
+                    sitepackages=False,
+                    usedevelop=False,
+                    deps=[(getdigest(""), "setuptools >= 35.0.2"), (getdigest(""), "wheel")],
+                    alwayscopy=False,
+                )
+            assert False  # pragma: no cover
 
     monkeypatch.setattr(CreationConfig, "readconfig", ProxyCurrentPython.readconfig)
 
+    # provide as Python the current python executable
     def venv_lookup(venv, name):
         assert name == "python"
+        venv.envconfig.envdir = py.path.local(sys.executable).join("..", "..")
         return sys.executable
 
     monkeypatch.setattr(VirtualEnv, "_venv_lookup", venv_lookup)
 
+    # don't allow overriding the tox config data for the host Python
+    def finish_venv(self):
+        return
+
+    monkeypatch.setattr(VirtualEnv, "finish", finish_venv)
+
+    # we lie that it's an environment with no packages in it
     @tox.hookimpl
     def tox_runenvreport(venv, action):
         return []
 
     monkeypatch.setattr(venv, "tox_runenvreport", tox_runenvreport)
+
+    # intercept the build session to save it and we intercept the popen invocations
+    prev_build = tox.session.build_session
+
+    def build_session(config):
+        res.session = prev_build(config)
+        res._popen = res.session.popen
+        monkeypatch.setattr(res.session, "popen", popen)
+        return res.session
+
+    monkeypatch.setattr(tox.session, "build_session", build_session)
+
+    # collect all popen calls
+    def popen(cmd, **kwargs):
+        # we don't want to perform installation of new packages, just replace with an always ok cmd
+        if "pip" in cmd and "install" in cmd:
+            cmd = ["python", "-c", "print({!r})".format(cmd)]
+        activity_id = res.session._actions[-1].id
+        activity_name = res.session._actions[-1].activity
+        try:
+            ret = res._popen(cmd, **kwargs)
+        except tox.exception.InvocationError as exception:  # pragma: no cover
+            ret = exception  # pragma: no cover
+        finally:
+            res.popens.append((activity_id, activity_name, kwargs.get("env"), ret, cmd))
+        return ret
+
+    return res
+
+
+@pytest.fixture(scope="session")
+def current_tox_py():
+    """generate the current (test runners) python versions key
+    e.g. py37 when running under Python 3.7"""
+    return "py{}".format("".join(str(i) for i in sys.version_info[0:2]))
